import { Component } from '@angular/core';
import { AbstractControl, FormControl, FormGroup, Validators } from '@angular/forms';
import { ColDef, ValueGetterParams } from 'ag-grid-community';
import { Job } from '../../../models/job.model';
import { JobGroup } from '../../../models/jobGroup.model';
import { NotificationService } from '../../../services/notification.service';
import { AdminService } from '../../services/admin.service';
import { BooleanCellRendererComponent } from '../boolean-cell-renderer/boolean-cell-renderer';
import { AttachedJobsComponent } from './attached-jobs/attached-jobs.component';
import { AttachedUsersComponent } from './attached-users/attached-users.component';
import { DeleteJobComponent } from './delete-job/delete-job.component';
import { DeleteJobsGroupComponent } from './delete-jobs-group/delete-jobs-group.component';
import { ValidateJobComponent } from './validate-job/validate-job.component';

@Component({
  selector: 'app-admin-manage-jobs',
  templateUrl: './manage-jobs.component.html',
  styleUrls: ['../../admin.scss'],
})
export class ManageJobsComponent {
  public newJobForm = new FormGroup({
    job: new FormControl('', [Validators.required]),
  });
  public newJobsGroupForm = new FormGroup({
    jobsGroup: new FormControl('', [Validators.required]),
  });

  public validatedJobs: Job[] = [];
  public unvalidatedJobs: Job[] = [];
  public jobsGroups: JobGroup[] = [];
  public deleteModalOpened = false;
  public validatedJobsNames: string[] = [];
  public mergeJobModalOpened = false;
  public jobToDelete: Job = null;
  public jobsGroupsNames: string[] = [];
  public deleteJobsGroupModalOpened = false;
  public jobsGroupToDelete: JobGroup = null;
  public jobsGroupToEdit: string = null;
  public mergeOpe: any = {
    newJob: {
      name: null,
      hasPersonalOffer: true,
      validated: false,
    },
    formerId: '',
  };
  public contextRow: any;

  public columnDefs: ColDef<Job>[];
  public columnDefsJobsGroups: ColDef<Job>[];
  public invalidJobsColumnDefs: ColDef<Job>[];
  public components;
  public defaultColDef: ColDef = {
    editable: true,
    sortable: true,
    flex: 1,
    minWidth: 100,
    filter: true,
    resizable: true,
  };
  public invalidColDef: ColDef = {
    ...this.defaultColDef,
    cellStyle: {
      color: '#da6c2e',
    },
  };

  constructor(
    private adminService: AdminService,
    private notificationService: NotificationService,
  ) {
    this.findValidatedJobs();
    this.findUnvalidatedJobs();
    this.getJobsGroupsWithJobs();
    this.initColumnDef();
    this.invalidJobsColumnDefs = [
      {
        headerName: 'Fonction',
        field: 'name',
        editable: true,
        onCellValueChanged: this.onChange.bind(this),
      },
      {
        headerName: 'Offre d’accompagnements',
        field: 'hasPersonalOffer',
        cellRenderer: 'booleanCellRenderer',
        cellEditor: 'agSelectCellEditor',
        valueGetter: this.translateBoolean,
        singleClickEdit: true,
        cellEditorParams: {
          values: [true, false],
          cellEditorPopup: true,
        },
        onCellValueChanged: this.onChange.bind(this),
      },
      {
        headerName: 'Acteurs rattachés',
        cellRenderer: 'attachedUsersComponent',
        cellRendererParams: {
          users: 'users',
        },
        minWidth: 500,
      },
      {
        headerName: 'Fusionner',
        field: '_toMergeWith',
        cellEditor: 'agSelectCellEditor',
        singleClickEdit: true,
        cellEditorParams: (): { values: string[] } => {
          return {
            values: ['', ...this.validatedJobsNames],
          };
        },
        onCellValueChanged: this.onMerge.bind(this),
      },
      {
        headerName: 'Valider',
        editable: false,
        minWidth: 100,
        cellRenderer: 'validateJobComponent',
        cellRendererParams: {
          onClick: this.onValidateButtonClick.bind(this),
          label: 'Valider',
        },
      },
    ];
    this.columnDefsJobsGroups = [
      {
        headerName: 'Nom',
        field: 'name',
        editable: true,
        onCellValueChanged: this.onEdit.bind(this),
        minWidth: 200,
      },
      {
        headerName: 'Fonctions rattachées',
        editable: false,
        cellRenderer: 'attachedJobsComponent',
        minWidth: 200,
      },
      {
        headerName: 'Suppression',
        editable: false,
        minWidth: 100,
        cellRenderer: 'deleteJobsGroupComponent',
        cellRendererParams: {
          onClick: this.onDeleteJobsGroupClick.bind(this),
          label: 'Supprimer',
        },
      },
    ];
    this.components = {
      booleanCellRenderer: BooleanCellRendererComponent,
      deleteJobComponent: DeleteJobComponent,
      deleteJobsGroupComponent: DeleteJobsGroupComponent,
      validateJobComponent: ValidateJobComponent,
      attachedJobsComponent: AttachedJobsComponent,
      attachedUsersComponent: AttachedUsersComponent,
    };
  }

  private initColumnDef(): void {
    this.columnDefs = [
      {
        headerName: 'Fonction',
        field: 'name',
        editable: true,
        onCellValueChanged: this.onChange.bind(this),
        minWidth: 200,
      },
      {
        headerName: 'Offre d’accompagnements',
        field: 'hasPersonalOffer',
        cellRenderer: 'booleanCellRenderer',
        cellEditor: 'agSelectCellEditor',
        valueGetter: this.translateBoolean,
        singleClickEdit: true,
        cellEditorParams: {
          values: [true, false],
        },
        onCellValueChanged: this.onChange.bind(this),
      },
      {
        headerName: 'Acteurs rattachés',
        cellRenderer: 'attachedUsersComponent',
        cellRendererParams: {
          users: 'users',
        },
        minWidth: 350,
      },
      {
        headerName: 'Groupe de fonctions',
        field: 'jobsGroup',
        minWidth: 100,
        cellEditor: 'agSelectCellEditor',
        singleClickEdit: true,
        cellEditorParams: (): { values: string[] } => {
          return {
            values: ['', ...this.jobsGroupsNames],
          };
        },
        valueGetter: (params) => params.data.jobsGroup?.name ?? '',
        valueSetter: (params): boolean => {
          params.data.jobsGroup = this.jobsGroups.find((jobsGroup) => jobsGroup.name === params.newValue);
          return true;
        },
        onCellValueChanged: this.onJobsGroupChange.bind(this),
      },
      {
        headerName: 'Suppression',
        editable: false,
        minWidth: 100,
        cellRenderer: 'deleteJobComponent',
        cellRendererParams: {
          onClick: this.onDeleteButtonClick.bind(this),
          label: 'Supprimer',
        },
      },
    ];
  }

  public onDeleteButtonClick(arg): void {
    this.deleteJob(arg.data, false);
  }

  get job(): AbstractControl<string, string> {
    return this.newJobForm.get('job');
  }

  get jobsGroup(): AbstractControl<string, string> {
    return this.newJobsGroupForm.get('jobsGroup');
  }

  public onSubmit(): void {
    if (this.newJobForm.valid) {
      const name: string = this.newJobForm.value.job;
      this.adminService.createJob(name).subscribe(
        () => {
          this.findValidatedJobs();
          this.newJobForm.reset();
          this.notificationService.showSuccess('Votre fonction a bien été créée');
        },
        (err) => {
          this.notificationService.showError(`${err.error.message}`, 'Une erreur est survenue');
        },
      );
    }
  }

  public onValidateButtonClick(arg): void {
    this.validateJob(arg.data._id);
  }

  public deleteJob(job: Job, shouldDelete: boolean): void {
    this.toggleDeleteJobModal(job);
    if (shouldDelete) {
      this.adminService.deleteJob(job._id).subscribe(
        (data) => {
          this.validatedJobs = this.validatedJobs.filter((obj) => obj._id !== data._id);
          this.notificationService.showSuccess('La fonction a bien été supprimée.');
        },
        (err) => {
          this.notificationService.showError(`${err.error.message}`, 'Une erreur est survenue');
        },
      );
    }
  }

  public onChange(arg): void {
    if (arg.oldValue === arg.newValue) {
      return;
    } else {
      this.adminService.updateJob(arg.data._id, arg.data).subscribe(() => {
        this.notificationService.showSuccess("L'opération a réussie.");
        this.findValidatedJobs();
        this.findUnvalidatedJobs();
      });
    }
  }

  public translateBoolean(params: ValueGetterParams): '✔️' | '❌' {
    return params.data.hasPersonalOffer ? '✔️' : '❌';
  }

  public onMerge(arg): void {
    if (arg.newValue) {
      const selectedJob = this.validatedJobs.find((job) => job.name === arg.newValue);
      this.toggleMergeJobModal(selectedJob, arg.data._id, arg);
    }
  }

  public mergeJob(newJobId: string, formerId: string, shouldMerge: boolean, context?): void {
    this.mergeJobModalOpened = !this.mergeJobModalOpened;
    if (shouldMerge) {
      this.adminService.mergeJob(formerId, newJobId).subscribe((data) => {
        context.node.data[context.colDef.field] = data;
        context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
        this.findValidatedJobs();
        this.findUnvalidatedJobs();
        this.notificationService.showSuccess("L'opération a réussie.");
      });
    } else {
      context.node.data[context.colDef.field] = context.oldValue;
      context.api.refreshCells({ rowNodes: [context.node], columns: [context.column.colId] });
    }
  }

  public validateJob(jobId: string): void {
    this.adminService.validateJob(jobId).subscribe(() => {
      this.findValidatedJobs();
      this.findUnvalidatedJobs();
    });
  }

  public toggleDeleteJobModal(jobToDelete: Job): void {
    this.jobToDelete = jobToDelete;
    this.deleteModalOpened = !this.deleteModalOpened;
  }

  public toggleMergeJobModal(jobToMerge: Job, formerId: string, dataGrid): void {
    this.mergeOpe.formerId = formerId;
    this.mergeOpe.newJob = jobToMerge;
    this.contextRow = dataGrid;
    this.mergeJobModalOpened = !this.mergeJobModalOpened;
  }

  public findValidatedJobs(): void {
    this.validatedJobsNames = [];
    this.adminService.getValidatedJobs().subscribe((jobs) => {
      this.validatedJobs = jobs;
      this.validatedJobsNames = jobs.map((job) => job.name);
      jobs.forEach((job) => {
        // The field: 'jobsGroup.name' need an empty string if no jobsGroup
        if (!job.jobsGroup) {
          job.jobsGroup = { name: '' };
        }
      });
    });
  }

  public findUnvalidatedJobs(): void {
    this.adminService.getUnvalidatedJobs().subscribe((jobs) => {
      this.unvalidatedJobs = jobs;
    });
  }

  public getJobsGroupsWithJobs(): void {
    this.jobsGroupsNames = [];
    this.adminService.getJobsGroupsWithJobs().subscribe((jobsGrps) => {
      this.jobsGroups = jobsGrps;
      this.jobsGroupsNames = jobsGrps.map((jobsGroup) => jobsGroup.name);
    });
  }

  public onSubmitNewJobsGroup(): void {
    if (!this.newJobsGroupForm.valid) {
      return;
    }
    const nameJobsGroup: string = this.newJobsGroupForm.value.jobsGroup;
    this.adminService.createJobsGroup(nameJobsGroup).subscribe(
      () => {
        this.getJobsGroupsWithJobs();
        this.newJobsGroupForm.reset();
        this.initColumnDef();

        this.notificationService.showSuccess('Le nouveau groupe de fonctions a bien été créé');
      },
      (err) => {
        this.notificationService.showError(`${err.error.message}`, 'Une erreur est survenue');
      },
    );
  }

  public onEdit(arg): void {
    if (arg.oldValue === arg.newValue) {
      return;
    } else {
      this.adminService.editJobsGroup(arg.data._id, arg.data.name).subscribe(() => {
        this.notificationService.showSuccess(
          `Le groupe de fonction ${arg.oldValue} a bien été remplacé par ${arg.data.name}.`,
        );
        this.getJobsGroupsWithJobs();
        this.initColumnDef();
      });
    }
  }

  public onJobsGroupChange(arg): void {
    const selectedJob = this.validatedJobs.find((job) => job._id === arg.data._id);
    const selectedJobsGroup = this.jobsGroups.find((jobsGroup) => jobsGroup.name === arg.newValue);
    selectedJob.jobsGroup = selectedJobsGroup;

    this.adminService.updateJob(arg.data._id, selectedJob).subscribe(() => {
      this.getJobsGroupsWithJobs();
      this.newJobsGroupForm.reset();

      if (!selectedJobsGroup) {
        this.notificationService.showSuccess(`Le groupe de fonctions a bien été enlevé.`);
      } else {
        this.notificationService.showSuccess(
          `Le groupe de fonctions ${selectedJobsGroup.name} a bien été attribuée à la fonction ${arg.data.name}.`,
        );
      }
    });
  }

  public toggleDeleteJobsGroupModal(jobsGroupToDelete: JobGroup): void {
    this.jobsGroupToDelete = jobsGroupToDelete;
    this.deleteJobsGroupModalOpened = !this.deleteJobsGroupModalOpened;
  }

  public onDeleteJobsGroupClick(arg): void {
    this.deleteJobsGroup(arg.data, false);
  }

  public deleteJobsGroup(jobGroup: JobGroup, shouldDelete: boolean): void {
    this.toggleDeleteJobsGroupModal(jobGroup);
    if (shouldDelete) {
      this.adminService.deleteJobsGroup(jobGroup._id).subscribe(
        () => {
          this.getJobsGroupsWithJobs();
          this.newJobsGroupForm.reset();
          this.initColumnDef();
          this.notificationService.showSuccess('Le groupe de fonctions a bien été supprimé.');
        },
        (err) => {
          this.notificationService.showError(`${err.error.message}`, 'Une erreur est survenue');
        },
      );
    }
  }

  public getUsersRowHeight(params): number {
    const users = params.data.users || [];
    if (users.length <= 1) return 40;
    return users.length * 40;
  }

  public getJobsRowHeight(params): number {
    const jobs = params.data.jobs || [];
    if (jobs.length <= 1) return 40;
    return jobs.length * 40;
  }
}
