import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { lastValueFrom, Observable } from 'rxjs';
import { DataType } from '../models/DataType.model';
import { Structure } from '../models/structure.model';
import { Filter } from '../structure-list/models/filter.model';
import { StructureService } from './structure.service';

@Injectable({
  providedIn: 'root',
})
export class DataService {
  public structures = [
    {
      name: 'Tous les lieux présents sur la cartographie Rés’in',
      slug: 'structures',
    },
    {
      name: 'Lieux proposant un accompagnement gratuit au numérique',
      slug: 'structures-accompagnement-gratuit',
    },
    {
      name: "Lieux labellisés 'Maison France Service'",
      slug: 'structures-france-service',
    },
    {
      name: "Lieux accueillant des Conseiller·es Numériques (dispositif de l'État)",
      slug: 'structures-conseillers-numeriques',
    },
    {
      name: 'Lieux proposant du matériel numérique en accès libre',
      slug: 'structures-materiel',
    },
  ];

  public users = [
    {
      name: 'Liste des accompagnants.es numériques',
      slug: 'membres-accompagnants-numeriques',
    },
    {
      name: 'Liste des Conseiller.es Numériques (dispositif de l’État)',
      slug: 'membres-conseillers-numeriques',
    },
  ];

  constructor(
    private http: HttpClient,
    private structureService: StructureService,
  ) {}

  /**
   * Retrieve pdf file
   */
  public getPdf(urlPath: string, fileName: string): Observable<Blob> {
    const body = {
      urlPath: urlPath,
      fileName: fileName,
    };

    return this.http.post('api/pdf', body, {
      responseType: 'blob',
    });
  }

  /**
   * Get structures using specified filter for this DataType
   */
  public getStructures(dataType: DataType, CTMs?: string[], inseeCodes?: string[]): Observable<Structure[]> {
    const filters: Filter[] = [];
    switch (dataType.slug) {
      // Structures
      case 'structures-accompagnement-gratuit':
        filters.push(new Filter('freeWorkShop', 'yes', null, true));
        filters.push(new Filter('freeWorkShop', 'underCondition', null, true));
        break;
      case 'structures-france-service':
        filters.push(new Filter('labelsQualifications', 'maisonFranceService', null));
        break;
      case 'structures-conseillers-numeriques':
        filters.push(new Filter('labelsQualifications', 'conseillerNumFranceServices', null));
        break;
      case 'structures-materiel':
        filters.push(new Filter('selfServiceMaterial', 'computer', null, true));
        filters.push(new Filter('selfServiceMaterial', 'printer', null, true));
        filters.push(new Filter('selfServiceMaterial', 'scanner', null, true));
        break;
    }

    // CTMs filters
    CTMs?.forEach((ctm) => {
      filters.push(new Filter('ctm', ctm, undefined, true));
    });

    return this.structureService.getStructures(filters, undefined, undefined, undefined, inseeCodes);
  }

  public async getUsers(dataType: DataType, ctms?: string[], inseeCodes?: string[]): Promise<any> {
    let hasPersonalOffer = false;
    let job = [];
    switch (dataType.slug) {
      case 'membres-accompagnants-numeriques':
        hasPersonalOffer = true;
        break;
      case 'membres-conseillers-numeriques':
        job = ['Conseiller numérique', 'Conseillère numérique'];
        break;
    }

    return lastValueFrom(
      this.http.get('/api/users/filteredUsers', {
        params: {
          hasPersonalOffer,
          job: job.join(','),
          ctms: ctms ? ctms.join(',') : '',
          inseeCodes: inseeCodes ? inseeCodes.join(',') : '',
        },
      }),
    );
  }
}
